#!/bin/bash
# --- saf.ai setup script. ---
# --- generated on Sun 14 Mar 2021 09:33:20 AM UTC by -masud ---

# This is not really something we would run into, however
# it is a good idea to simply stop this in its tracks!
# Right now we do not support Illumos or OpenSolaris but one day! :-)
if [ "$KSH_VERSION" = 'Version JM 93t+ 2010-03-05' ]; then
	# The version of ksh93 that ships with many illumos systems does not
	# support the "local" extension. Print a message rather than fail in
	# subtle ways later on:
	echo 'saf.ai does not work with this ksh93 version; please try bash!' >&2
	exit 1
fi

set -u # treat unset vars as an error

# Common shell functions.

die() {
	local rv=$1
	shift
	(printf "%s\n" "[ERROR] $@") 1>&2
	exit $rv
}

warn() {
	(echo "[WARNING] $@") 1>&2
}

info() {
	(echo "[INFO] $@") 1>&2
}

BUG() {
	local line TRACE

	for i in 9 8 7 6 5 4 3 2 1 0; do
		line=$(caller $i)
		if [ -z ${line:+X} ]; then
			continue
		fi
		line="FROM ${line}"
		TRACE=$(echo -e "${TRACE:+${TRACE}\n...}$line")
	done
cat 1>&2 <<EOF
[BUG]:: $@
	You have discovered an internal problem with cefs package-manager
	Please help us correct it by filing a report, or sending us an email to
	support@saf.ai!
--- Trace ---
$TRACE
--- End of trace
EOF
	exit 127
}

is_func() {
	local a=$(type -t "$1") 
	test ${a:-X} = "function"
}

need_cmd() {
	if ! check_cmd "$1" > /dev/null 2>&1; then
		die 2 "need '$1' (command not found)"
	fi
}

check_cmd() {
	command -v "$1"
}

log() {
	case "$1" in
	INIT)
		if mkdir -p $(dirname ${LOGFILE}) ; then 
			echo "Using ${LOGFILE}" 1>&2
			exec 5>${LOGFILE}
		else
			exec 5>&2
		fi
		;;
	EXEC)
		shift
		echo "# $@" >&5
		"$@" 1>&5 2>&5
		;;
	*)
		echo "$@" 1>&5
		;;
	esac
}

# Run a command that should never fail. If the command fails execution
# will immediately terminate with an error showing the failing
# command.
ensure() {
	if ! await_status "$@" ; then
		die 9 "Failed to execute\n\t$*\n(see $LOGFILE for details)"
	fi
}

status() {
	case "$1" in
	MSG)
		shift
		local msg="$@"
		printf "%s" "$msg"
		# eval is needed by bash
		if [[ ${#msg} -lt 70 ]]; then
			eval printf ".%.0s" {1..$((70 - ${#msg}))}
		fi 
		;;
	OK|FAIL|SKIP)
		printf "[ %-6s ]" $1
		;;
	END)
		printf "\n"
		;;
	esac
}

await_status() {
	if [ "$1" = "-m" ]; then
		msg="$2"
		status MSG $msg
		shift; shift
	elif is_func $1:msg ; then
		status MSG $($1:msg $*)
	else
		status MSG "$1"
	fi

	if [ $# -eq 0 ]; then
		BUG "await_status used with out a command"
	fi

	if log EXEC "$@" ; then status OK ; else status FAILED; fi

	status END
}

init_vars() {
	P="${0}"
	P=$(realpath ${P})
	D=$(dirname ${0})
	## --- Configuration variables ---

	export prefix=${prefix:-/usr}
	export bindir=${prefix}/bin
	export libdir=${prefix}/lib
	export sysconfdir=/etc/cefs
	export localstatedir=/var/run/cefs
	export libexecdir=${prefix}/libexec
	export syslibdir=/var/lib/cefs
	export logdir=/var/log/cefs
	export innerrootdir=root
	export installroot=${installroot:-${prefix}}
	export PACKAGE_NAME=cefs
	export PACKAGE_VERSION=1.0.3
	export TARBALL_URL=https://saf.ai/downloads/stable/cefs-1.0.3.tar
	export TARBALL_BASENAME=cefs.tar
	export SETUP_SH=/home/masud/projects/rust-saf.ai/package-manager/_build/stable/build/cefs-init.sh

	## --- Configuration variables: END ---

	return 0
}

#!/bin/bash
# https://github.com/pollev/bash_progress_bar - See license at end of file

# Usage:
# Source this script
# enable_trapping <- optional to clean up properly if user presses ctrl-c
# setup_scroll_area <- create empty progress bar
# draw_progress_bar 10 <- advance progress bar
# draw_progress_bar 40 <- advance progress bar
# block_progress_bar 45 <- turns the progress bar yellow to indicate some action is requested from the user
# draw_progress_bar 90 <- advance progress bar
# destroy_scroll_area <- remove progress bar

# Constants
CODE_SAVE_CURSOR="\033[s"
CODE_RESTORE_CURSOR="\033[u"
CODE_CURSOR_IN_SCROLL_AREA="\033[1A"
COLOR_FG="\e[30m"
COLOR_BG="\e[42m"
COLOR_BG_BLOCKED="\e[43m"
RESTORE_FG="\e[39m"
RESTORE_BG="\e[49m"

# Variables
PROGRESS_BLOCKED="false"
TRAPPING_ENABLED="false"
TRAP_SET="false"

CURRENT_NR_LINES=0

setup_scroll_area() {
	# If trapping is enabled, we will want to activate it whenever we setup the scroll area and remove it when we break the scroll area
	if [ "$TRAPPING_ENABLED" = "true" ]; then
		trap_on_interrupt
	fi

	lines=$(tput lines)
	CURRENT_NR_LINES=$lines
	let lines=$lines-1
	# Scroll down a bit to avoid visual glitch when the screen area shrinks by one row
	echo -en "\n"

	# Save cursor
	echo -en "$CODE_SAVE_CURSOR"
	# Set scroll region (this will place the cursor in the top left)
	echo -en "\033[0;${lines}r"

	# Restore cursor but ensure its inside the scrolling area
	echo -en "$CODE_RESTORE_CURSOR"
	echo -en "$CODE_CURSOR_IN_SCROLL_AREA"

	# Start empty progress bar
	draw_progress_bar 0
}

destroy_scroll_area() {
	lines=$(tput lines)
	# Save cursor
	echo -en "$CODE_SAVE_CURSOR"
	# Set scroll region (this will place the cursor in the top left)
	echo -en "\033[0;${lines}r"

	# Restore cursor but ensure its inside the scrolling area
	echo -en "$CODE_RESTORE_CURSOR"
	echo -en "$CODE_CURSOR_IN_SCROLL_AREA"

	# We are done so clear the scroll bar
	clear_progress_bar

	# Scroll down a bit to avoid visual glitch when the screen area grows by one row
	echo -en "\n\n"

	# Once the scroll area is cleared, we want to remove any trap previously set. Otherwise, ctrl+c will exit our shell
	if [ "$TRAP_SET" = "true" ]; then
		trap - INT
	fi
}

draw_progress_bar() {
	percentage=$1
	lines=$(tput lines)
	let lines=$lines

	# Check if the window has been resized. If so, reset the scroll area
	if [ "$lines" -ne "$CURRENT_NR_LINES" ]; then
		setup_scroll_area
	fi

	# Save cursor
	echo -en "$CODE_SAVE_CURSOR"

	# Move cursor position to last row
	echo -en "\033[${lines};0f"

	# Clear progress bar
	tput el

	# Draw progress bar
	PROGRESS_BLOCKED="false"
	print_bar_text $percentage

	# Restore cursor position
	echo -en "$CODE_RESTORE_CURSOR"
}

block_progress_bar() {
	percentage=$1
	lines=$(tput lines)
	let lines=$lines
	# Save cursor
	echo -en "$CODE_SAVE_CURSOR"

	# Move cursor position to last row
	echo -en "\033[${lines};0f"

	# Clear progress bar
	tput el

	# Draw progress bar
	PROGRESS_BLOCKED="true"
	print_bar_text $percentage

	# Restore cursor position
	echo -en "$CODE_RESTORE_CURSOR"
}

clear_progress_bar() {
	lines=$(tput lines)
	let lines=$lines
	# Save cursor
	echo -en "$CODE_SAVE_CURSOR"

	# Move cursor position to last row
	echo -en "\033[${lines};0f"

	# clear progress bar
	tput el

	# Restore cursor position
	echo -en "$CODE_RESTORE_CURSOR"
}

print_bar_text() {
	local percentage=$1
	local cols=$(tput cols)
	let bar_size=$cols-17

	local color="${COLOR_FG}${COLOR_BG}"
	if [ "$PROGRESS_BLOCKED" = "true" ]; then
		color="${COLOR_FG}${COLOR_BG_BLOCKED}"
	fi

	# Prepare progress bar
	let complete_size=($bar_size*$percentage)/100
	let remainder_size=$bar_size-$complete_size
	progress_bar=$(echo -ne "["; echo -en "${color}"; printf_new "#" $complete_size; echo -en "${RESTORE_FG}${RESTORE_BG}"; printf_new "." $remainder_size; echo -ne "]");

	# Print progress bar
	echo -ne " Progress ${percentage}% ${progress_bar}"
}

enable_trapping() {
	TRAPPING_ENABLED="true"
}

trap_on_interrupt() {
	# If this function is called, we setup an interrupt handler to cleanup the progress bar
	TRAP_SET="true"
	trap cleanup_on_interrupt INT
}

cleanup_on_interrupt() {
	destroy_scroll_area
	exit
}

printf_new() {
	str=$1
	num=$2
	v=$(printf "%-${num}s" "$str")
	echo -ne "${v// /$str}"
}


# SPDX-License-Identifier: MIT
#
# Copyright (c) 2018--2020 Polle Vanhoof
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
usage() {
	cat 1>&2 <<EOF
cefs 1.0.3 package-manager
The setup & installation manager for cefs

USAGE:
	# cefs-init.sh [FLAGS] [OPTIONS]

FLAGS:
	-v, --verbose			Enable progress output
	-q, --quiet				Disable progress output (default)
	-y						Agree to all confirmation messages
	-h, --help				Show this help message
	-k, --keep				Do not delete any temporory files

OPTIONS:
	-o, --output-file=FILE 				log output to FILE
	-o, --append-output=FILE 			append output to FILE
EOF
}

main() {

	if [ $(id -u) != "0" ]; then
		die 1 "You must be root to install cefs"
	fi

	init_vars

	BASEDIR=${TMPDIR:-/tmp}/${PACKAGE_NAME}/${PACKAGE_VERSION}
	EXTRACT_DIR=${BASEDIR}/extract-$$
	LOGFILE=${localstatedir}/$PACKAGE_NAME-$PACKAGE_VERSION-install.$$.log

	for arg in "$@"; do
		case "$arg" in
		-h|--help)
			usage
			exit 0
			;;
		-y|--yes)
			confirmed="-y"
			;;

		-V|--version)
			show_version="y"
			;;

		-k)
			keep=1
			;;
		-v)
			verbosity=$((${verbosity:-0}+1))
		esac
	done

	trap "cleanup" 0 1 2 3 15

	log INIT

	need_cmd mkdir
	need_cmd rm
	need_cmd cat
	need_cmd which
	need_cmd tar

	setup_scroll_area

	mkdir -p ${BASEDIR} ${EXTRACT_DIR}
	pushd ${BASEDIR}

	ensure download $TARBALL_URL ${TARBALL_BASENAME} && draw_progress_bar 20
	ensure deps && draw_progress_bar 40
	ensure unpack && draw_progress_bar 60
	ensure after_unpack && draw_progress_bar 80
	draw_progress_bar 100 && echo "${PACKAGE_NAME} ${PACKAGE_VERSION} installed successfully!"
	popd 
}

# TODO: Add a download only flag
# check for existing archive with SHA256 and don't download if exists.
# maybe provide rsync service

download:msg() {
	echo "Downloading $2"
}

download() {
	[ -z ${1:+X} ] && BUG "No URL provided."
	local url="$1"
	local save_to=${2:-$(basename $1)}

	local dlexe=$(check_cmd curl || check_cmd wget || die 1 "Need curl or wget, in order to get the latest package")
	case $dlexe in
		*curl)
			dlopts="-o-"
			;;
		*wget)
			dlopts="-o-"
			;;
	esac
	echo using $dlexe to download $url saving to $save_to
	$dlexe $dlopts "$1" > ${save_to} || die 2 "unable to download ${url}"
	# export DLNAME=$(basename "$1")
	# mv ${DLNAME} ${TARBALL_BASENAME}
}

# ---- saf.ai unpacker: BEGINS ----
# unpack assumes that an archive is embedded in the script
# between ^__ARCHIVE_BEGINS__ and ^__ARCHIVE_ENDS__

deps:msg() {
	echo "Installing dependencies"
}

deps() {
	sudo apt install -q -y	\
		nilfs-tools			\
		libfuse3-dev		\
		xz-utils			\
		libssl-dev			\
		fuse3				\
		daemon 				\
		unzip
	return 0
}

unpack:msg() {
	echo "Unpacking ${2:-${PACKAGE_NAME}} (this takes some time, please be patient)"
}

unpack() {
	rm -rf ${EXTRACT_DIR}
	mkdir -p ${EXTRACT_DIR}
	tar -xf ${BASEDIR}/${TARBALL_BASENAME} -C ${EXTRACT_DIR}
}

after_unpack:msg() {
	echo "Finish unpacking"
}

after_unpack() {
	if [ -x ${EXTRACT_DIR}/post_extract.sh ]; then
		${EXTRACT_DIR}/post_extract.sh
	fi

	if [ -x ${EXTRACT_DIR}/install.sh ] ; then
		${EXTRACT_DIR}/install.sh

	elif [ -d ${EXTRACT_DIR}/${innerrootdir} ]; then
		src=${EXTRACT_DIR}/${innerrootdir}/
		if [ -d ${src}/$(basename ${prefix}) ] ; then
			src=${src}/$(basename ${prefix})
		fi
		(cd $src && cp -v -a . ${prefix})
	else
		BUG "Invalid package setup, please contact support@saf.ai"
	fi
}

cleanup:msg() {
	echo "Cleaning up"
}

cleanup() {
	[ -z ${keep:+Z} ] && rm -rf ${EXTRACT_DIR}
	destroy_scroll_area
	exit 0
}

main "$@"
