#!/bin/bash

# Common shell functions.

die() {
	local rv=$1
	shift
	(printf "%s\n" "[ERROR] $@") 1>&2
	exit $rv
}

warn() {
	(echo "[WARNING] $@") 1>&2
}

info() {
	(echo "[INFO] $@") 1>&2
}

BUG() {
	local line TRACE

	for i in 9 8 7 6 5 4 3 2 1 0; do
		line=$(caller $i)
		if [ -z ${line:+X} ]; then
			continue
		fi
		line="FROM ${line}"
		TRACE=$(echo -e "${TRACE:+${TRACE}\n...}$line")
	done
cat 1>&2 <<EOF
[BUG]:: $@
	You have discovered an internal problem with cefs package-manager
	Please help us correct it by filing a report, or sending us an email to
	support@saf.ai!
--- Trace ---
$TRACE
--- End of trace
EOF
	exit 127
}

is_func() {
	local a=$(type -t "$1") 
	test ${a:-X} = "function"
}

need_cmd() {
	if ! check_cmd "$1" > /dev/null 2>&1; then
		die 2 "need '$1' (command not found)"
	fi
}

check_cmd() {
	command -v "$1"
}

log() {
	case "$1" in
	INIT)
		if mkdir -p $(dirname ${LOGFILE}) ; then 
			echo "Using ${LOGFILE}" 1>&2
			exec 5>${LOGFILE}
		else
			exec 5>&2
		fi
		;;
	EXEC)
		shift
		echo "# $@" >&5
		"$@" 1>&5 2>&5
		;;
	*)
		echo "$@" 1>&5
		;;
	esac
}

# Run a command that should never fail. If the command fails execution
# will immediately terminate with an error showing the failing
# command.
ensure() {
	if ! await_status "$@" ; then
		die 9 "Failed to execute\n\t$*\n(see $LOGFILE for details)"
	fi
}

status() {
	case "$1" in
	MSG)
		shift
		local msg="$@"
		printf "%s" "$msg"
		# eval is needed by bash
		if [[ ${#msg} -lt 70 ]]; then
			eval printf ".%.0s" {1..$((70 - ${#msg}))}
		fi 
		;;
	OK|FAIL|SKIP)
		printf "[ %-6s ]" $1
		;;
	END)
		printf "\n"
		;;
	esac
}

await_status() {
	if [ "$1" = "-m" ]; then
		msg="$2"
		status MSG $msg
		shift; shift
	elif is_func $1:msg ; then
		status MSG $($1:msg $*)
	else
		status MSG "$1"
	fi

	if [ $# -eq 0 ]; then
		BUG "await_status used with out a command"
	fi

	if log EXEC "$@" ; then status OK ; else status FAILED; fi

	status END
}

init_vars() {
	P="${0}"
	P=$(realpath ${P})
	D=$(dirname ${0})
	## --- Configuration variables ---

	export prefix=${prefix:-/usr}
	export bindir=${prefix}/bin
	export libdir=${prefix}/lib
	export sysconfdir=/etc/cefs
	export localstatedir=/var/run/cefs
	export libexecdir=${prefix}/libexec
	export syslibdir=/var/lib/cefs
	export logdir=/var/log/cefs
	export innerrootdir=root
	export installroot=${installroot:-${prefix}}
	export PACKAGE_NAME=cefs
	export PACKAGE_VERSION=1.0.3
	export TARBALL_URL=https://saf.ai/downloads/stable/cefs-1.0.3.tar
	export TARBALL_BASENAME=cefs.tar
	export SETUP_SH=/home/masud/projects/rust-saf.ai/package-manager/_build/stable/build/cefs-init.sh

	## --- Configuration variables: END ---

	return 0
}


woops() {
	if [ -z ${DEVICE:+X} ]; then
    	cat <<EOF
cefs recover" 1>&2
	unable to find cefs on $DEVICE
EOF
exit 1
	fi
}

usage() {
	cat <<EOF
Usage is:

    cefs recover list <store device>              - show recovery points
    cefs recover <store device> id# [ path ]      - recover [path] from $id

    cefs recover stores                       - show recoverable storages

EOF
}


make_available() {
	dev=$1
	num=$2
	name=$(echo ${dev} | sed -e s,^/,,  -e s,/,_,g)
	restoredir=${syslibdir}/by-source/${name}/recall/$num
	sudo mkdir -p $restoredir
	echo -ne "Making memory $num of $dev available at\n\t$restoredir\n..."
	chcp ss $dev $num || die 8 "Unable to make a permanent memory for $dev:$num"
	mount -o cp=$num,ro $dev $restoredir || die 9 "Unable to make memory $dev:$num available"
	echo Ready
}

main () {

	if [ -z ${1:+Z} ] ; then
    		usage
    		exit 0
	fi

	init_vars

	LOGFILE=${logdir}/recovery.log

	log INIT

	eval STORES=$(awk \
		'BEGIN { count=0; printf "(" } END { printf ")" } /^cefs:/ { printf "[" count "]=" substr($1,6) ":" $2; count++ }' \
		/proc/mounts)

	case $1 in
	/dev/*)
		if [ -z ${2:+X} ]; then
			die 2 "Please provide an instance to recover from"
		fi
		dev=$(grep cefs:$1 /proc/mounts | awk '{print substr($1,6)  ":" $2 }')
		mp=$(echo $dev | cut -f2 -d:)
		dev=$(echo $dev | cut -f1 -d:)
		make_available $dev $2
		;;

	list)
		shift
		if [ -z ${1:+Z} ]; then
			usage
			exit 0
		fi
		dev=$1
		if ! ( lscp ${dev} | sed -e '/CNO/d' | awk '{
			printf("%8d - %s at %s -- [%s]\n", $1, $2, $3, $4 == "ss" ? "permanent" : "transient" ) } END { printf "\n" }' )
		then
			die 2 "Unable to find any data about $dev, are you sure it's a CEFS filesystem?"
		fi
		;;
	stores)
		for s in $STORES; do
			dev=$(echo $s | cut -d: -f1)
			mp=$(echo $s | cut -d: -f2)
			echo "CEFS $dev mounted on $mp"
		done
		;;
	esac


}

main "$@" || exit 1

